# Generated by Django 3.0.11 on 2021-01-27 20:21

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_audit_events', '0003_auto_20201218_1113'),
    ]

    operations = [
        migrations.AlterField(
            model_name='archivedauditevent',
            name='url',
            field=models.URLField(blank=True, max_length=1000, null=True, verbose_name='URL'),
        ),
        migrations.AlterField(
            model_name='auditevent',
            name='url',
            field=models.URLField(blank=True, max_length=1000, null=True, verbose_name='URL'),
        ),
    ]
